@@ -30,10 +30,10 @@ func EnsureDir(dir string, mode fs.FileMode) error {
 	}
 
 	if fi, err := fs.Stat("."); err == nil {
-		// Apprently the stat may fail even though the mkdirall passed. If it
+		// Apparently the stat may fail even though the mkdirall passed. If it
 		// does, we'll just assume things are in order and let other things
 		// fail (like loading or creating the config...).
-		currentMode := fi.Mode() & 0777
+		currentMode := fi.Mode() & 0o777
 		if currentMode != mode {
 			err := fs.Chmod(".", mode)
 			// This can fail on crappy filesystems, nothing we can do about it.
@@ -63,6 +63,8 @@ func DefaultConfig(path string, myID protocol.DeviceID, evLogger events.Logger,
 
 	if skipPortProbing {
 		l.Infoln("Using default network port numbers instead of probing for free ports")
+		// Record address override initially
+		newCfg.GUI.RawAddress = newCfg.GUI.Address()
 	} else if err := newCfg.ProbeFreePorts(); err != nil {
 		return nil, err
 	}
@@ -83,7 +85,7 @@ func DefaultConfig(path string, myID protocol.DeviceID, evLogger events.Logger,
 }
 
 // LoadConfigAtStartup loads an existing config. If it doesn't yet exist, it
-// creates a default one, without the default folder if noDefaultFolder is ture.
+// creates a default one, without the default folder if noDefaultFolder is true.
 // Otherwise it checks the version, and archives and upgrades the config if
 // necessary or returns an error, if the version isn't compatible.
 func LoadConfigAtStartup(path string, cert tls.Certificate, evLogger events.Logger, allowNewerConfig, noDefaultFolder, skipPortProbing bool) (config.Wrapper, error) {
@@ -139,7 +141,7 @@ func copyFile(src, dst string) error {
 		return err
 	}
 
-	if err := os.WriteFile(dst, bs, 0600); err != nil {
+	if err := os.WriteFile(dst, bs, 0o600); err != nil {
 		// Attempt to clean up
 		os.Remove(dst)
 		return err
